<?php
/**
 * Created by PhpStorm.
 * User: Jaskolek
 * Date: 2016-12-13
 * Time: 13:09
 */

namespace Scraper;


use GuzzleHttp\Client;
use Monolog\Logger;
use Symfony\Component\DomCrawler\Crawler;

class NatuerlichScraper
{
    const PAGE_ENCODING = "UTF-8";
//    const PAGE_ENCODING = "Windows-1252";
    const PER_PAGE = 12;
    /**
     * @var Client
     */
    protected $client;

    /**
     * @var Logger
     */
    protected $logger;

    /**
     * NatuerlichScraper constructor.
     * @param Client $client
     * @param Logger $logger
     */
    public function __construct(Client $client, Logger $logger)
    {
        $this->client = $client;
        $this->logger = $logger;
    }

    public function fetchSelectList(Crawler $crawler)
    {
        $selectList = [];
        $crawler->filter("form select")->each(function (Crawler $crawler) use (&$selectList) {
            $name = $crawler->attr("name");
            $optionList = [];
            $crawler->filter("option")->each(function (Crawler $crawler) use (&$optionList) {
                $value = $crawler->attr("value");
                $key = $crawler->text();
                if ($value != 0) {
                    $optionList[$key] = new Option($key, $value);
                }
            });
            $selectList[$name] = $optionList;
        });

        return $selectList;
    }

    public function scrape()
    {
        $this->logger->debug("Starting scraping...");
        $this->client->get("https://natuerlich.de");
        $body = $this->client->get("https://natuerlich.de");

        $crawler = new Crawler();
        $crawler->addHtmlContent($body, self::PAGE_ENCODING);

        $root = new NatuerlichCategory(0, "root", "https://natuerlich.de/damenschuhe/");
        $this->updateNatuerlichCategoryChildList($root);

        $categoryBreadcrumbList = $this->getCategoryDescendantList($root);
        $leafList = array_filter($categoryBreadcrumbList, function (NatuerlichCategory $category) {
            return count($category->getChildList()) == 0;
        });
        foreach($leafList as $leaf){
            echo $leaf->getBreadcrumbAsString() . " ---- " . $leaf->getName() . "\n";
        }


        //map: url => list of categories
        $urlCategoryBreadcrumbList = [];

        foreach ($leafList as $leaf) {
            $productUrlList = $this->fetchProductListFromCategory($leaf);
            foreach($productUrlList as $url){
                if(!isset($urlCategoryBreadcrumbList)){
                    $urlCategoryBreadcrumbList[$url] = [];
                }
                $urlCategoryBreadcrumbList[$url][] = $leaf->getBreadcrumbAsString();
            }
        }

        $productList = [];
        foreach($urlCategoryBreadcrumbList as $url => $categoryBreadcrumbList){
            try {
                $product = $this->fetchProductFromUrl($url);
                $product->categoryBreadcrumb = $categoryBreadcrumbList;
                $productList[$product->id] = $product;
            }catch (\Exception $e){
                $this->logger->error("Can not scrape $url. " . $e->getMessage() );
            }
        }

        return $productList;
    }

    /**
     * @param $productUrl
     * @return Product
     */
    public function fetchProductFromUrl($productUrl)
    {
        $this->logger->debug("Scraping $productUrl");
        $body = $this->client->get($productUrl)->getBody()->getContents();

        $crawler = new Crawler();
        $crawler->add($body, self::PAGE_ENCODING);

        $product = new Product();

        $product->name = trim($crawler->filter("h1.product--title")->text());
        $product->description = trim($crawler->filter(".product--description")->html());
        $product->price = $crawler->filter("[itemprop='price']")->attr("content");
        $product->imageUrl = $crawler->filter(".image--box .image--element")->attr("data-img-original");
        $properties = [];
        $crawler->filter(".product--properties-row")->each(function(Crawler $crawler)use(&$properties){
            $name = substr(trim($crawler->filter(".product--properties-label")->text()), 0, -1);
            $value = trim($crawler->filter(".product--properties-value")->text());
            $properties[$name] = $value;
        });


        $product->color = $properties["Farbe"];
        $product->material = $properties["Material"];
        $labelImgNode = $crawler->filter(".product--supplier-link img");
        if($labelImgNode->count() > 0) {
            $product->label = $labelImgNode->attr("alt");
        }else{
            $product->label = null;
        }
        $widthNameList = explode(", ", $properties["Weite"]);
        $mainSoleWidthName = array_shift($widthNameList);
        //fetching for first width;

        $mainSoleWidth = $this->fetchSoleData($body);
        $mainSoleWidth->width = $mainSoleWidthName;
        $groupNodeList = $crawler->filter(".variant--group");

        if($groupNodeList->count() == 2){
            $groupNodeList->last()->filter(".variant--option")->each(function(Crawler $crawler, $i)use($productUrl, &$product){
//                if($i == 0) return;
                $value = $crawler->filter("input")->attr("value");
                $name = trim($crawler->filter("label")->text());
                $variantUrl = $productUrl . "?group[2]=$value&template=ajax";

                $body = $this->client->get($variantUrl)->getBody()->getContents();
                $soleWidth = $this->fetchSoleData($body);
                $soleWidth->width = $name;
                $soleWidth->id = (int) substr($soleWidth->id, 0, -3);
                $product->soleWidthList[$soleWidth->id] = $soleWidth;
            });
        }else{
            $product->soleWidthList[$mainSoleWidth->id] = $mainSoleWidth;
        }

        $product->id = implode("-", array_keys($product->soleWidthList));
        return $product;
    }

    public function fetchSoleData($body)
    {
        $crawler = new Crawler();
        $crawler->add($body, self::PAGE_ENCODING);

        $soleWidth = new SoleWidth();
        $soleWidth->id = trim($crawler->filter(".entry--content")->text());
        $soleWidth->sizeList = [];
        $crawler->filter(".variant--group")->first()->filter(".variant--option")->each(function(Crawler $crawler)use(&$soleWidth){
            $isDisabled = ($crawler->filter("input")->attr("disabled") == "disabled");
            $size = trim($crawler->filter(".option--label")->text());

            $soleWidth->sizeList[$size] = !$isDisabled;
        });

        return $soleWidth;
    }

    /**
     * @param NatuerlichCategory|null $category
     * @return NatuerlichCategory[]
     */
    public function getCategoryDescendantList(NatuerlichCategory $category)
    {
        $descendantList = $category->getChildList();
        foreach ($category->getChildList() as $child) {
            foreach ($this->getCategoryDescendantList($child) as $descendant) {
                $descendantList[] = $descendant;
            }
        }
        return $descendantList;
    }

    public function updateNatuerlichCategoryChildList(NatuerlichCategory $category, $deep = 0)
    {
        $this->logger->debug("Fetching category tree... " . $category->getName());
        $body = $this->client->get($category->url)->getBody()->getContents();

        $crawler = new Crawler();
        $crawler->addHtmlContent($body, self::PAGE_ENCODING);

        $selector = ".sidebar--categories-navigation .is--level$deep > li > a";
        /** @var NatuerlichCategory[] $subCategoryList */
        $subCategoryList = $crawler->filter($selector)->each(function (Crawler $crawler) {
            $id = $crawler->attr("data-categoryid");
            $url = $crawler->attr("href");
            $name = $crawler->attr("title");

            return new NatuerlichCategory($id, $name, $url);
        });

        $subCategoryList = array_filter($subCategoryList);
        foreach ($subCategoryList as $subCategory) {
            $subCategory->setParent($category);
            $this->updateNatuerlichCategoryChildList($subCategory, $deep + 1);
//            if(rand(0,5) > 3) break;
        }
    }

    public function fetchProductListFromCategory(NatuerlichCategory $category)
    {
        $this->logger->debug("Doing search...");

        $response = $this->client->get("https://natuerlich.de/widgets/listing/listingCount/sCategory/" . $category->getId());
        $data = $response->json();
        $totalCount = $data["totalCount"];
        $pageCount = ceil($totalCount / self::PER_PAGE);

        $totalProductUrlList = [];
        for ($page = 1; $page <= $pageCount; ++$page) {
            $productUrlList = $this->fetchProductListPageFromCategory($category, $page);
            $totalProductUrlList = array_merge($productUrlList, $totalProductUrlList);
        }

        return $totalProductUrlList;
    }

    /**
     * @param NatuerlichCategory $category
     * @param $page
     * @return array
     */
    public function fetchProductListPageFromCategory(NatuerlichCategory $category, $page)
    {
        $this->logger->debug("Fetching page $page for " . $category->getBreadcrumbAsString());

        $url = "https://natuerlich.de/widgets/Listing/ajaxListing?p=$page&mode=next&c=" . $category->getId();

        $body = $this->client->get($url)->getBody()->getContents();
        $crawler = new Crawler();
        $crawler->addHtmlContent($body, self::PAGE_ENCODING);

        $productUrlList = $crawler->filter(".product--image")->each(function(Crawler $crawler){
            $url = preg_replace('/\?.*/', '', $crawler->attr("href"));
            return $url;
        });

        return $productUrlList;
    }
}