<?php
/**
 * Created by PhpStorm.
 * User: jasko
 * Date: 12.01.2017
 * Time: 14:04
 */

namespace Ordering;

use \GuzzleHttp\Client;
use Monolog\Logger;

class OrderManager
{
    /**
     * @var Logger
     */
    protected $logger;

    /**
     * @var NatuerlichApi
     */
    protected $natuerlichApi;

    /**
     * @var OrderResultRepository
     */
    protected $orderResultRepository;

    /**
     * @var Config
     */
    protected $config;
    /**
     * @var JoomlaApi[]
     */
    protected $joomlaApiList = [];

    /**
     * OrderManager constructor.
     * @param Logger $logger
     * @param NatuerlichApi $natuerlichApi
     * @param OrderResultRepository $orderResultRepository
     * @param Config $config
     */
    public function __construct(Logger $logger, NatuerlichApi $natuerlichApi, OrderResultRepository $orderResultRepository, Config $config)
    {
        $this->logger = $logger;
        $this->natuerlichApi = $natuerlichApi;
        $this->orderResultRepository = $orderResultRepository;
        $this->config = $config;
    }


    /**
     * @param $hikashopList Hikashop[]
     */
    public function run($hikashopList)
    {
        /** @var JoomlaOrder[] $orderList */
        $orderList = [];
        foreach ($hikashopList as $hikashop) {
            $this->logger->info("Fetching orders from " . $hikashop->url);
            $orderList = array_merge($orderList, $this->fetchOrderList($hikashop));
        }

        $filteredOrderList = [];
        foreach ($orderList as $order) {
            $result = $this->orderResultRepository->find($order->getUniqueId());
            if ($result == null) {
                $filteredOrderList[] = $order;
            } else {
                $this->logger->debug("Order " . $order->orderNumber . " - " . $order->hikashop->name . " already processed!");
            }
        }

        $this->natuerlichApi->getCsrfToken();
        $resultList = $this->makeOrder($filteredOrderList);

        foreach ($resultList as $result) {
            if ($result->success) {
                $this->logger->addDebug("Updating order status " . $result->getId());
                $joomlaApi = $this->getJoomlaApi($result->order->hikashop);
                $joomlaApi->updateOrder($result->order);
            }
        }
        $this->orderResultRepository->insertMany($resultList);
    }

    /**
     * @param $orderList
     * @return OrderResult[]
     */
    protected function makeOrder($orderList)
    {

        $this->natuerlichApi->logIn(
            $this->config->get("natuerlichUsername"),
            $this->config->get("natuerlichPassword")
        );


        $resultList = [];
        for ($i = 0; $i < count($orderList); ++$i) {
            $this->logger->debug($i);
            $order = $orderList[$i];
            try {
                $this->addToCart($order);
            } catch (\Exception $e) {
                $this->logger->debug($e->getMessage());
                //remove everything from cart and try again
                unset($orderList[$i]);
                $orderList = array_values($orderList);
                $resultList[] = new OrderResult(false, $order, $e->getMessage());
                $i = -1;

                $this->natuerlichApi->clearCart();
            }
        }
        foreach ($orderList as $order) {
            $resultList[] = new OrderResult(true, $order, "");
        }

        $price = $this->natuerlichApi->getCartPrice();
        $minPrice = (float)str_replace(",", ".", $this->config->get("minOrderPrice"));

        if ($price < $minPrice) {
            //if price is too small, just return [] and do noting
            $this->logger->info("Total price is $price" . "EUR. Min price is $minPrice" . "EUR. NOT ORDERING!");
            return [];
        }

        try {
            $this->natuerlichApi->makeOrder();
        }catch (\Exception $e){
            $this->logger->error("Can not make order! " . $e->getMessage());
            /** @var OrderResult $result */
            foreach($resultList as $result){
                if($result->success){
                    $result->success = false;
                    $result->message = $e->getMessage();
                }
            }
        }
        return $resultList;
    }

    protected function addToCart(JoomlaOrder $order)
    {
        foreach ($order->joomlaOrderProductList as $joomlaOrderProduct) {
            $this->logger->info("Ordering " . $joomlaOrderProduct->id . ", quantity: " . $joomlaOrderProduct->quantity);
            $this->natuerlichApi->orderJoomlaOrderProduct($joomlaOrderProduct);
        }
    }


    public function fetchOrderList(Hikashop $hikashop)
    {
        $api = $this->getJoomlaApi($hikashop);

        if (!$api->isLoggedIn()) {
            $api->logIn($hikashop->username, $hikashop->password);
        }

        $orderList = $api->fetchOrderList();
        foreach ($orderList as $order) {
            $order->hikashop = $hikashop;
        }

        return $orderList;
    }

    public function getJoomlaApi($hikashop)
    {
        $key = spl_object_hash($hikashop);
        if (!isset($this->joomlaApiList[$key])) {
            $defaultHeaders = [
                "Accept" => "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8",
                "Accept-Encoding" => "gzip, deflate, sdch, br",
                "Accept-Language" => "pl-PL,pl;q=0.8,en-US;q=0.6,en;q=0.4",
                "Cache-Control" => "max-age=0",
                "Connection" => "keep-alive",
                "Upgrade-Insecure-Requests" => "1",
                "User-Agent" => "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/55.0.2883.87 Safari/537.36",
            ];
            $joomlaClient = new Client([
                "base_url" => $hikashop->url,
                "defaults" => [
                    "verify" => false,
                    "cookies" => true,
                    "headers" => $defaultHeaders
//                    "debug" => true
                ]
            ]);
            $this->joomlaApiList[$key] = new JoomlaApi($joomlaClient);
        }
        return $this->joomlaApiList[$key];
    }
}